! cellpot.f90

program cellpot
    implicit none

    integer, parameter :: m = 6
    double precision, parameter :: dt = 0.05d0
    double precision, parameter :: totalTime = 600.0d0

    double precision :: Iinj = -2000.0d0  !; %2000.0;  %(pA)
    double precision :: Ton  = 50.0d0
    double precision :: Toff = 300.0d0
    double precision :: vinit = -63.27d0
    double precision, parameter :: vtraub = -55.0d0 ! (mV)

    double precision, dimension(m) :: y


    double precision :: t
    integer :: i, npts
    
    npts = floor(totalTime/dt)+1
    call setInitialValues(y)

    open(unit = 11, file='ax.txt', form='formatted', access='stream', status='replace')
    do i = 1, npts
        t = dt*i
        write(unit=11, fmt='(2E15.7)') t, y(1)
        call advanceStep(t, y)
    end do
    close(unit=11)
    call system("../../tools/gc ax.txt ");

contains

    subroutine advanceStep(tt, y)
        double precision, intent(in) :: tt
        double precision, dimension(:), intent(inout) :: y 
        double precision, dimension(m) :: yn, k1, k2, k3, k4

        call rhs(tt, y, k1)
        yn = y + dt * k1/2.0d0
        
        call rhs(tt+dt/2.0d0, yn, k2)
        yn = y + dt*k2/2.0d0
        
        call rhs(tt+dt/2.0d0, yn, k3)
        yn = y + dt*k3

        call rhs(tt+dt, yn, k4);
        y = y + dt*(k1 + 2.0d0*k2 + 2.0d0*k3 + k4)/6.0d0
    end subroutine advanceStep


    subroutine get_Na_m(v, a, b)
        double precision, intent(in) :: v
        double precision, intent(out) :: a, b
        double precision :: v2
        v2 = v - vtraub
        a = 0.32d0 * (13.0d0-v2) / ( exp((13.0d0-v2)/4.0d0) - 1.0d0)
        b = 0.28d0 * (v2-40.0d0) / ( exp((v2-40.0d0)/5.0d0) - 1.0d0)
    end subroutine get_Na_m

    subroutine get_Na_h(v, a, b)
        double precision, intent(in) :: v
        double precision, intent(out) :: a, b
        double precision :: v2
        v2 = v - vtraub
        a = 0.128d0 * exp((17.0d0-v2)/18.0d0)
        b = 4.0d0/(1.0d0 + exp((40.0d0-v2)/5.0d0))
    end subroutine get_Na_h

    subroutine get_K_n(v, a, b)
        double precision, intent(in) :: v
        double precision, intent(out) :: a, b
        double precision :: v2
        v2 = v - vtraub;
        a = 0.032d0 * (15.0d0-v2)/( exp((15.0d0-v2)/5.0d0) - 1.0d0)
        b = 0.5d0 * exp((10.0d0-v2)/40.0d0);
    end subroutine get_K_n

    subroutine get_TCa_m(v, tau, inf)
        double precision, intent(in) :: v
        double precision, intent(out) :: tau, inf
        double precision :: vshift=0, vs
        vs = v + vshift
        tau = (3.0d0 + 1.0d0 / ( exp((vs+25.0d0)/10.0d0) + exp(-(vs+100.0d0)/15.0d0)))
        inf = 1.0d0 / ( 1.0d0 + exp(-(vs+50.0d0)/7.4d0))
    end subroutine get_TCa_m

    subroutine get_TCa_h(v, tau, inf)
        double precision, intent(in) :: v
        double precision, intent(out) :: tau, inf
        double precision :: vshift=0, vs
        
        vs = v + vshift
        tau = ( 85.0d0 + 1.0d0 / ( exp((vs+46.0d0)/4.0d0) + exp(-(vs+405.0d0)/50.0d0) ) ) 
        inf = 1.0d0 / ( 1.0d0 + exp((vs+78.0d0)/5.0d0) )
    end subroutine get_TCa_h

    subroutine setInitialValues(y)
        double precision, intent(inout), dimension(:) :: y
        double precision :: v, a, b, tau, inf
        y(1) = vinit
        v = y(1)
        call get_Na_m(v, a, b)
        y(2) = a/(a+b)

        call get_Na_h(v, a, b)
        y(3) = a/(a+b)

        call get_K_n(v, a, b)
        y(4) = a/(a+b)

        call get_TCa_m(v, tau, inf)
        y(5) = inf

        call get_TCa_h(v, tau, inf)
        y(6) = inf
    end subroutine setInitialValues


    subroutine rhs (tt, y, dy)
        double precision, parameter :: Dm = 60.0d0      ! (micrometer) diameter of the cell
        double precision, parameter :: Lm = 70.0d0      ! (micrometer) length of the cell

        double precision, parameter :: dGleak = 0.002d0 ! (mS/cm^2)
        double precision, parameter :: dGNa   = 0.05d0  ! 0.53;   % (mS/cm^2)
        double precision, parameter :: dGK    = 0.15d0  ! 0.13;  % (mS/cm^2)
        double precision, parameter :: dGTCa  = 0.03d0  ! (mS/cm^2)
        double precision, parameter :: fac = 10.0d0     ! conversion factor (mS/cm^2) -> (pS/micrometer^2)
        
        double precision, parameter :: Am    = 3.14d0 * Dm * Lm  ! (squre_micrometer) surface area
        double precision, parameter :: Gleak = Am * dGleak * fac
        double precision, parameter :: Cmm   =  0.01d0 * Am
        double precision, parameter :: InvCm = 1.0d0/Cmm
        double precision, parameter :: GNa   = Am * dGNa * fac
        double precision, parameter :: GK    = Am * dGK * fac
        double precision, parameter :: GTCa  = Am * dGTCa * fac

        double precision, parameter :: Eleak = -65.0d0  ! (mV)
        double precision, parameter :: ENa   = 50.0d0   ! (mV)
        double precision, parameter :: EK    = -100.0d0 ! (mV)
        double precision, parameter :: ECa   = 50.0d0   ! (mV)

        double precision, intent(in) :: tt
        double precision, dimension(:), intent(in):: y
        double precision, dimension(:), intent(out):: dy
        double precision :: v, c
        double precision :: a, b, tau, inf
        double precision :: INa, IK, ITCa

        if(tt<Ton .or. t>=Toff)then
            c = 0.0d0
        else
            c = Iinj
        end if
        
        v = y(1)

        INa = GNa * y(2)**3 * y(3) * (v-ENa) 
        IK  = GK * y(4)**4 * (v-EK)     
        ITCa = GTCa * y(5)**2 * y(6) * (v - ECa)

        dy(1) = InvCm * (c - Gleak * (v-Eleak) - INa - IK - ITCa )
        call get_Na_m(v, a, b)
        dy(2) = a - (a+b) * y(2)

        call get_Na_h(v, a, b)
        dy(3) = a - (a+b) * y(3)

        call get_K_n(v, a, b)
        dy(4) = a - (a+b) * y(4)

        call get_TCa_m(v, tau, inf)
        dy(5) = (inf - y(5)) / tau

        call get_TCa_h(v, tau, inf)
        dy(6) = (inf - y(6)) / tau
    end subroutine rhs

end program cellpot

 