% cellpot.m
function cellpot
    tic
    m = 6;
    dt = 0.05;
    totalTime = 600.0;
    npts = int32(totalTime/dt)+1;

    Dm = 60.0;      % (micrometer) diameter of the cell
    Lm = 70.0;      % (micrometer) length of the cell

    dGleak = 0.002; % (mS/cm^2)
    dGNa   = 0.05;  % 0.53;   % (mS/cm^2)
    dGK    = 0.15;  % 0.13;  % (mS/cm^2)
    dGTCa  = 0.03;  % (mS/cm^2)
    fac = 10;       % conversion factor (mS/cm^2) -> (pS/micrometer^2)

    Eleak = -65.0;  % (mV)
    ENa   = 50.0;   % (mV)
    EK    = -100.0; % (mV)
    ECa   = 50.0;   % (mV)
    vtraub = -55.0; % (mV)

    Am    = 3.14 * Dm * Lm;    % (squre_micrometer) surface area
    Gleak = Am * dGleak * fac;
    Cmm   =  0.01 * Am;
    InvCm = 1/Cmm;
    GNa   = Am * dGNa * fac;
    GK    = Am * dGK * fac;
    GTCa  = Am * dGTCa * fac;

    Iinj = -2000; %2000.0;  %(pA)
    Ton  = 50;
    Toff = 300;   % 40;
    vinit = -63.27;

    y = zeros(1,m);
    y = setInitialValue(y);
    
    t = linspace(0.0, totalTime, npts);
    [t,y] = ode45(@(tt, yy)RHS_eqs(tt, yy), t, y);

    th = -20.0;
    sn = 0;
    for i=1:size(y,1)-1
        if y(i+1,1)>=th && y(i,1)<th
            sn = sn +1;
        end
    end
    fprintf('number of spikes = %d\n', sn)
    toc
    plot(t, y(:,1))
    
    function [a,b] = get_Na_m(v)
        v2 = v - vtraub;
        a = 0.32 * (13.0-v2) / ( exp((13.0-v2)/4.0) - 1.0);
        b = 0.28 * (v2-40.0) / ( exp((v2-40.0)/5.0) - 1.0);
    end

    function [a, b] = get_Na_h(v)
        v2 = v - vtraub;
        a = 0.128 * exp((17.0-v2)/18.0);
        b = 4.0/(1.0 + exp((40.0-v2)/5.0));
    end

    function [a,b] = get_K_n(v)
        v2 = v - vtraub;
        a = 0.032 * (15.0-v2)/( exp((15.0-v2)/5.0) - 1.0);
        b = 0.5 * exp((10.0-v2)/40);
    end

    function [tau, inf] = get_TCa_m(v)
        vshift=0;
        vs = v + vshift;
        tau = (3.0 + 1.0 / ( exp((vs+25.0)/10) + exp(-(vs+100.0)/15.0) ) );
        inf = 1.0 / ( 1.0 + exp(-(vs+50.0)/7.4) );
    end

    function [tau, inf] = get_TCa_h(v)
        vshift=0;
        vs = v + vshift;
        tau = ( 85.0 + 1.0 / ( exp((vs+46.0)/4.0) + exp(-(vs+405.0)/50.0) ) ) ;
        inf = 1.0 / ( 1.0 + exp((vs+78.0)/5.0) );
    end

    function y = setInitialValue(y)
        y(1) = vinit;
        v = y(1);
        [a,b] = get_Na_m(v);    y(2) = a/(a+b);
        [a,b] = get_Na_h(v);    y(3) = a/(a+b);
        [a,b]= get_K_n(v);      y(4) = a/(a+b);
        [~,inf] = get_TCa_m(v); y(5) = inf;
        [~,inf] = get_TCa_h(v); y(6) = inf;
    end


    function dy = RHS_eqs(t, y)
        if t<Ton || t>=Toff
            c = 0.0;
        else
            c = Iinj;
        end
        v = y(1);
        INa = GNa*y(2)^3*y(3)*(v-ENa);  % INa = GNa * m^3 * h * (v-Ena)
        IK  = GK * y(4)^4 * (v-EK);     % IK = GK * n^4 (v-EK)
        ITCa = GTCa * y(5)^2 * y(6) * (v - ECa);

        [a1,b1] = get_Na_m(v);
        [a2,b2] = get_Na_h(v);
        [a3,b3] = get_K_n(v);
        [tau1,inf1] = get_TCa_m(v);
        [tau2,inf2] = get_TCa_h(v);

        dy = [ InvCm * (c - Gleak * (v-Eleak) - INa - IK - ITCa );
            a1 - (a1+b1)*y(2);
            a2 - (a2+b2)*y(3);
            a3 - (a3+b3)*y(4);
            (inf1 - y(5)) / tau1;
            (inf2 - y(6)) / tau2];
    end
end
